referrerpolicy=no-referrer-when-downgrade

TickAction

Enum TickAction 

pub enum TickAction<AccountId, Balance, RelayBlockNumber> {
    SellRegion {
        owner: AccountId,
        paid: Balance,
        region_id: RegionId,
        region_end: Timeslice,
    },
    RenewRegion {
        owner: AccountId,
        renewal_id: PotentialRenewalId,
    },
    Refund {
        amount: Balance,
        who: AccountId,
    },
    ProcessAutoRenewals {
        after_timeslice: Timeslice,
        next_renewal_at: Timeslice,
    },
    SaleRotated {
        old_sale: MarketSaleInfo<RelayBlockNumber>,
        new_sale: MarketSaleInfo<RelayBlockNumber>,
    },
}
Outcome of Market::tick.

When pallet-broker calls Market::tick, it receives a list of TickActions which will be executed in order.

These actions are not executed by the market itself as they don’t fall into the scope of the market logic(e.g., transferring balances, updating region ownership). Instead, the market relies on pallet-broker to execute them.

Variants



SellRegion

Sell a region to an account.

Fields

owner: AccountId

New owner.

paid: Balance

Total price paid.

region_id: RegionId

Region identifier.

region_end: Timeslice

End of the region.



RenewRegion

Renew an existing region.

Fields

owner: AccountId

Current owner.

renewal_id: PotentialRenewalId

Renewal identifier.



Refund

Refund previously locked balance.

Fields

amount: Balance

Amount to return.

who: AccountId

Recipient.



ProcessAutoRenewals

Process the auto renewals which are stored in pallet-broker.

Fields

after_timeslice: Timeslice

Only auto-renewals allowing renewals after this timeslice should be processed.

next_renewal_at: Timeslice

When the next auto-renewal of this core can be made.



SaleRotated

Indicates that a new sale cycle has started.

This allows pallet-broker to handle sale boundary transitions.

Fields

old_sale: MarketSaleInfo<RelayBlockNumber>

Previously active sale.

new_sale: MarketSaleInfo<RelayBlockNumber>

Newly active sale.