pub trait CoreRangeProvider {
// Required method
fn core_range() -> Option<SoldCoresRange>;
}Expand description
Provides information about the range of cores that can be sold on a market.
Required Methods
Sourcefn core_range() -> Option<SoldCoresRange>
fn core_range() -> Option<SoldCoresRange>
Returns the range of core indices that can be sold on a market.
Returns None if the range is unknown (e.g., the CoreRangeProvider
implementer is not initialized).
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.